@@ -7,6 +7,19 @@ import operation_tracker
 _ = lambda x: x
 
 
+package_loader = PackageLoader('flask.ext.mongoengine', 'templates')
+
+
+def _maybe_patch_jinja_loader(jinja_env):
+    """Patch the jinja_env loader to include flaskext.mongoengine
+    templates folder if necessary.
+    """
+    if not isinstance(jinja_env.loader, ChoiceLoader):
+        jinja_env.loader = ChoiceLoader([jinja_env.loader, package_loader])
+    elif package_loader not in jinja_env.loader.loaders:
+        jinja_env.loader.loaders.append(package_loader)
+
+
 class MongoDebugPanel(DebugPanel):
     """Panel that shows information about MongoDB operations (including stack)
 
@@ -16,13 +29,8 @@ class MongoDebugPanel(DebugPanel):
     has_content = True
 
     def __init__(self, *args, **kwargs):
-        """
-        We need to patch jinja_env loader to include flask.ext.mongoengine
-        templates folder.
-        """
         super(MongoDebugPanel, self).__init__(*args, **kwargs)
-        self.jinja_env.loader = ChoiceLoader([self.jinja_env.loader,
-                          PackageLoader('flask.ext.mongoengine', 'templates')])
+        _maybe_patch_jinja_loader(self.jinja_env)
         operation_tracker.install_tracker()
 
     def process_request(self, request):
