@@ -5,11 +5,11 @@ from gettext import gettext as _
 import json
 
 from wtforms import widgets
-from wtforms.fields import SelectFieldBase, TextAreaField
+from wtforms.fields import SelectFieldBase, TextAreaField, StringField
 from wtforms.validators import ValidationError
 
 from mongoengine.queryset import DoesNotExist
-
+from mongoengine.python_support import txt_type, bin_type
 
 __all__ = (
     'ModelSelectField', 'QuerySetSelectField',
@@ -46,20 +46,24 @@ class QuerySetSelectField(SelectFieldBase):
         if self.allow_blank:
             yield (u'__None', self.blank_text, self.data is None)
 
-        if self.queryset == None:
+        if self.queryset is None:
             return
 
         self.queryset.rewind()
         for obj in self.queryset:
             label = self.label_attr and getattr(obj, self.label_attr) or obj
-            yield (obj.id, label, obj == self.data)
+            if isinstance(self.data, list):
+                selected = obj in self.data
+            else:
+                selected = self._is_selected(obj)
+            yield (obj.id, label, selected)
 
     def process_formdata(self, valuelist):
         if valuelist:
             if valuelist[0] == '__None':
                 self.data = None
             else:
-                if self.queryset == None:
+                if self.queryset is None:
                     self.data = None
                     return
 
@@ -75,6 +79,9 @@ class QuerySetSelectField(SelectFieldBase):
             if not self.data:
                 raise ValidationError(_(u'Not a valid choice'))
 
+    def _is_selected(self, item):
+        return item == self.data
+
 
 class QuerySetSelectMultipleField(QuerySetSelectField):
 
@@ -98,6 +105,9 @@ class QuerySetSelectMultipleField(QuerySetSelectField):
                 if not len(self.data):
                     self.data = None
 
+    def _is_selected(self, item):
+        return item in self.data
+
 
 class ModelSelectField(QuerySetSelectField):
     """
@@ -118,7 +128,6 @@ class ModelSelectMultipleField(QuerySetSelectMultipleField):
         super(ModelSelectMultipleField, self).__init__(label, validators, queryset=queryset, **kwargs)
 
 
-
 class JSONField(TextAreaField):
     def _value(self):
         if self.raw_data:
@@ -139,3 +148,24 @@ class DictField(JSONField):
         super(DictField, self).process_formdata(value)
         if value and not isinstance(self.data, dict):
             raise ValueError(self.gettext(u'Not a valid dictionary.'))
+
+
+class NoneStringField(StringField):
+    """
+    Custom StringField that counts "" as None
+    """
+
+    def process_formdata(self, valuelist):
+        if valuelist:
+            self.data = valuelist[0]
+        if self.data == "":
+            self.data = None
+
+class BinaryField(TextAreaField):
+    """
+    Custom TextAreaField that converts its value with bin_type.
+    """
+
+    def process_formdata(self, valuelist):
+        if valuelist:
+            self.data = bin_type( valuelist[0] )
