@@ -1,11 +1,20 @@
 """
 Tools for generating forms based on mongoengine Document schemas.
 """
+import decimal
+from bson import ObjectId
 from operator import itemgetter
+
+try:
+    from collections import OrderedDict
+except ImportError:
+    # Use bson's SON implementation instead
+    from bson import SON as OrderedDict
+
 from wtforms import fields as f, validators
 from mongoengine import ReferenceField
 
-from flask.ext.mongoengine.wtf.fields import ModelSelectField, ModelSelectMultipleField, DictField
+from flask.ext.mongoengine.wtf.fields import ModelSelectField, ModelSelectMultipleField, DictField, NoneStringField, BinaryField
 from flask.ext.mongoengine.wtf.models import ModelForm
 
 __all__ = (
@@ -20,10 +29,8 @@ def converts(*args):
     return _inner
 
 
-
-class ModelConverter():
+class ModelConverter(object):
     def __init__(self, converters=None):
-
         if not converters:
             converters = {}
 
@@ -38,7 +45,7 @@ class ModelConverter():
     def convert(self, model, field, field_args):
         kwargs = {
             'label': getattr(field, 'verbose_name', field.name),
-            'description': '',
+            'description': field.help_text or '',
             'validators': [],
             'filters': [],
             'default': field.default,
@@ -51,8 +58,13 @@ class ModelConverter():
         else:
             kwargs['validators'].append(validators.Optional())
 
+        ftype = type(field).__name__
+
         if field.choices:
             kwargs['choices'] = field.choices
+
+            if ftype in self.converters:
+                kwargs["coerce"] = self.coerce(ftype)
             if kwargs.pop('multiple', False):
                 return f.SelectMultipleField(**kwargs)
             return f.SelectField(**kwargs)
@@ -88,20 +100,20 @@ class ModelConverter():
             if kwargs.pop('password'):
                 return f.PasswordField(**kwargs)
         if field.max_length:
-            return f.TextField(**kwargs)
+            return f.StringField(**kwargs)
         return f.TextAreaField(**kwargs)
 
     @converts('URLField')
     def conv_URL(self, model, field, kwargs):
         kwargs['validators'].append(validators.URL())
         self._string_common(model, field, kwargs)
-        return f.TextField(**kwargs)
+        return NoneStringField(**kwargs)
 
     @converts('EmailField')
     def conv_Email(self, model, field, kwargs):
         kwargs['validators'].append(validators.Email())
         self._string_common(model, field, kwargs)
-        return f.TextField(**kwargs)
+        return NoneStringField(**kwargs)
 
     @converts('IntField')
     def conv_Int(self, model, field, kwargs):
@@ -131,7 +143,7 @@ class ModelConverter():
         #TODO: may be set file field that will save file`s data to MongoDB
         if field.max_bytes:
             kwargs['validators'].append(validators.Length(max=field.max_bytes))
-        return f.TextAreaField(**kwargs)
+        return BinaryField(**kwargs)
 
     @converts('DictField')
     def conv_Dict(self, model, field, kwargs):
@@ -144,12 +156,15 @@ class ModelConverter():
         if field.field.choices:
             kwargs['multiple'] = True
             return self.convert(model, field.field, kwargs)
-        unbound_field = self.convert(model, field.field, {})
-        kwargs = {
+        field_args = kwargs.pop("field_args", {})
+        unbound_field = self.convert(model, field.field, field_args)
+        unacceptable = {
             'validators': [],
             'filters': [],
+            'min_entries': kwargs.get('min_entries', 0)
         }
-        return f.FieldList(unbound_field, min_entries=0, **kwargs)
+        kwargs.update(unacceptable)
+        return f.FieldList(unbound_field, **kwargs)
 
     @converts('SortedListField')
     def conv_SortedList(self, model, field, kwargs):
@@ -170,6 +185,7 @@ class ModelConverter():
         kwargs = {
             'validators': [],
             'filters': [],
+            'default': field.default or field.document_type_obj,
         }
         form_class = model_form(field.document_type_obj, field_args={})
         return f.FormField(form_class, **kwargs)
@@ -182,10 +198,20 @@ class ModelConverter():
     def conv_GenericReference(self, model, field, kwargs):
         return
 
+    def coerce(self, field_type):
+        coercions = {
+            "IntField": int,
+            "BooleanField": bool,
+            "FloatField": float,
+            "DecimalField": decimal.Decimal,
+            "ObjectIdField": ObjectId
+        }
+        return coercions.get(field_type, unicode)
+
 
 def model_fields(model, only=None, exclude=None, field_args=None, converter=None):
     """
-    Generate a dictionary of fields for a given Django model.
+    Generate a dictionary of fields for a given database model.
 
     See `model_form` docstring for description of parameters.
     """
@@ -200,11 +226,11 @@ def model_fields(model, only=None, exclude=None, field_args=None, converter=None
     field_names = map(itemgetter(0), sorted(names, key=itemgetter(1)))
 
     if only:
-        field_names = (x for x in field_names if x in only)
+        field_names = set((x for x in only if x in set(field_names)))
     elif exclude:
-        field_names = (x for x in field_names if x not in exclude)
+        field_names = set((x for x in set(field_names) if x not in exclude))
 
-    field_dict = {}
+    field_dict = OrderedDict()
     for name in field_names:
         model_field = model._fields[name]
         field = converter.convert(model, model_field, field_args.get(name))
